'''
Aufgabe 2
'''
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import numpy as np
import math
import pgm


def createLowPassFilter(w, h, a, d0):
    
    result = np.zeros((h, w))
    
    for y in xrange(h):
        for x in xrange(w):
            y2 = y - h / 2
            x2 = x - w / 2
            d = math.sqrt(y2 * y2 + x2 * x2)
            result[y, x] = a + (1 - a) * math.exp(-d * d / (2 * d0 * d0))
    return result



def createBLPF(w, h, n, d0):
    
    result = np.zeros((h, w))
    
    for y in xrange(h):
        for x in xrange(w):
            y2 = y - h / 2
            x2 = x - w / 2
            d = math.sqrt(y2 * y2 + x2 * x2)
            result[y, x] = 1 / (1 + math.pow(d/d0, 2 * n))
    return result



def createHeuristicWienerFilter(filter, k):
    
    h, w = filter.shape
    result = np.zeros_like(filter)
    
    for y in xrange(h):
        for x in xrange(w):
            h = filter[y, x]
            h2 = h.real*h.real + h.imag*h.imag
            result[y, x] = 1/h * h2/(h2 + k)
    return result



def applyFilter(image, filter):
    
    result = image * filter   
    return result



def getAmplitudeImage(image):
    
    result = np.array([[math.sqrt(v.real * v.real + v.imag * v.imag) for v in row] for row in image])
    result = np.log(1 + result)
    return result





def aufgabe3a():
    
    image = pgm.imread("Schriftzug.pgm")
    
    imageFT = np.fft.fft2(image)
    imageFTShift = np.fft.fftshift(imageFT)
    imageFTAmp = getAmplitudeImage(imageFTShift)
    
    filter = 1 / createLowPassFilter(image.shape[1], image.shape[0], 0.001, 20)
    filterAmp = getAmplitudeImage(filter)
    
    resultFT = applyFilter(imageFTShift, filter)
    resultFTAmp = getAmplitudeImage(resultFT)
    
    result = np.fft.ifftshift(resultFT)
    result = np.fft.ifft2(result)
    result = result.real
    
    plt.figure(1)
    plt.subplot(241)
    plt.title("Original F")
    plt.imshow(image, cmap=plt.cm.gray, norm=colors.NoNorm())
    plt.subplot(242)
    plt.title("F - FT")
    plt.imshow(imageFTAmp, cmap=plt.cm.gray)
    plt.subplot(243)
    plt.title("G")
    plt.imshow(filterAmp, cmap=plt.cm.gray)
    plt.subplot(244)
    plt.title("F * G")
    plt.imshow(resultFTAmp, cmap=plt.cm.gray)
    plt.subplot(245)
    plt.title("F * G - IFT")
    plt.imshow(result, cmap=plt.cm.gray)
    plt.show()   
 
 
 
def aufgabe3b():
    
    image = pgm.imread("Schriftzug.pgm")
    
    imageFT = np.fft.fft2(image)
    imageFTShift = np.fft.fftshift(imageFT)
    imageFTAmp = getAmplitudeImage(imageFTShift)
    
    filter = 1 / createLowPassFilter(image.shape[1], image.shape[0], 0.001, 20)
    filterAmp = getAmplitudeImage(filter)
    
    filter2 = createBLPF(image.shape[1], image.shape[0], 10, 65)
    filter2Amp = getAmplitudeImage(filter2)
    
    filterCombined = filter2 * filter
    filterCombinedAmp = getAmplitudeImage(filterCombined)
    
    resultFT = applyFilter(imageFTShift, filterCombined)
    resultFTAmp = getAmplitudeImage(resultFT)
    
    result = np.fft.ifftshift(resultFT)
    result = np.fft.ifft2(result)
    result = result.real
    
    plt.figure(1)
    plt.subplot(241)
    plt.title("Original F")
    plt.imshow(image, cmap=plt.cm.gray, norm=colors.NoNorm())
    plt.subplot(242)
    plt.title("F - FT")
    plt.imshow(imageFTAmp, cmap=plt.cm.gray)
    plt.subplot(243)
    plt.title("G")
    plt.imshow(filterAmp, cmap=plt.cm.gray)
    plt.subplot(244)
    plt.title("H")
    plt.imshow(filter2Amp, cmap=plt.cm.gray)
    plt.subplot(245)
    plt.title("G * H")
    plt.imshow(filterCombinedAmp, cmap=plt.cm.gray)
    plt.subplot(246)
    plt.title("F * (G * H)")
    plt.imshow(resultFTAmp, cmap=plt.cm.gray)
    plt.subplot(247)
    plt.title("F * (G * H) - IFT")
    plt.imshow(result, cmap=plt.cm.gray)
    plt.show()   
    
    
    
def aufgabe3c():
    
    image = pgm.imread("Schriftzug.pgm")
    
    imageFT = np.fft.fft2(image)
    imageFTShift = np.fft.fftshift(imageFT)
    imageFTAmp = getAmplitudeImage(imageFTShift)
    
    filter = createLowPassFilter(image.shape[1], image.shape[0], 0.001, 20)
    filter = createHeuristicWienerFilter(filter, 0.00006)
    filterAmp = getAmplitudeImage(filter)
    
    resultFT = applyFilter(imageFTShift, filter)
    resultFTAmp = getAmplitudeImage(resultFT)
    
    result = np.fft.ifftshift(resultFT)
    result = np.fft.ifft2(result)
    result = result.real
    
    plt.figure(1)
    plt.subplot(241)
    plt.title("Original F")
    plt.imshow(image, cmap=plt.cm.gray, norm=colors.NoNorm())
    plt.subplot(242)
    plt.title("F - FT")
    plt.imshow(imageFTAmp, cmap=plt.cm.gray)
    plt.subplot(243)
    plt.title("G")
    plt.imshow(filterAmp, cmap=plt.cm.gray)
    plt.subplot(244)
    plt.title("F * G")
    plt.imshow(resultFTAmp, cmap=plt.cm.gray)
    plt.subplot(245)
    plt.title("F * G - IFT")
    plt.imshow(result, cmap=plt.cm.gray)
    plt.show()     
    

#aufgabe3a()
#aufgabe3b()
aufgabe3c()
